//----------------------------------------------------------------------
//MODEULE: led.c
//COMMON DESCRIPTION:
//  This module contain procedures for led-indicator control, using
//one-wire protocol.
//----------------------------------------------------------------------
//#include "led_mlan.h"
//
#ifdef LED_MLAN_ENABLED
//
#include "progmem.h"
// Digit images for seven-segment indicators (store in PROGRAM-FLASH memory)
unsigned char Digit[] __attribute__ ((progmem))={0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,
                                                 0x7F,0x6F,0x77,0x7C,0x39,0x5E,0x79,0x71};
// Temporary array of led data
unsigned char TLED[0x04]; 
// Counter for flash
unsigned char flashcnt;
unsigned char ledflash;
unsigned char ledsenable;
//----------------------------------------------------------------------
// Put word to indicator in hex format
void printhexw(unsigned int w,unsigned char led[])
  {signed char i;
   for(i=3;i>=0;i--)
     {led[i]=PRG_RDB(Digit+(w & 0x000F));
      w=w >> 0x04;
     }
  }
//----------------------------------------------------------------------
// Put word to indicator in dec format
void printdecw(unsigned int w,unsigned char led[])
  {unsigned char i;
   unsigned char d;
   unsigned int p=1000;
   for(i=0;i<4;i++)
     {d=w/p;
      led[i]=PRG_RDB(Digit+d);
      w=w-(p*d);
      p=p/10;
     }
  }
//----------------------------------------------------------------------
// Put word to indicator in dec format and first any symbol  SDDD 
void printsym3d(unsigned char sym,unsigned int w,unsigned char led[])
  {unsigned char i;
   unsigned char d;
   unsigned int p=100;
   for(i=0;i<3;i++)
     {d=w/p;
      led[i+1]=PRG_RDB(Digit+d);
      w=w-(p*d);
      p=p/10;
     }
   led[0]=sym;
  }
//----------------------------------------------------------------------
// Switch on led number n
void ledon(unsigned char l,unsigned char led[])
  {if(l<0x20)
     {led[l>>3]|=(1 << (l & 0x07));}
  }
//----------------------------------------------------------------------
// Switch off led number n
void ledoff(unsigned char l,unsigned char led[])
  {if(l<0x20)
     {led[l>>3]&=~(1 << (l & 0x07));}
  }
//----------------------------------------------------------------------
// Enable flash mode for led n
void ledflashon(unsigned char l,unsigned char led[])
  {if(l<0x20)
     {led[(l>>3)+4]|=(1 << (l & 0x07));}
  }
//----------------------------------------------------------------------
// Disable flash mode for led n
void ledflashoff(unsigned char l,unsigned char led[])
  {if(l<0x20)
     {led[(l>>3)+4]&=~(1 << (l & 0x07));}
  }
//----------------------------------------------------------------------
// Init led module
void led_init(){}
// Enable automatic refresh leds
void led_start(){ledsenable=1;}
// Disable automatic refresh leds
void led_stop(){ledsenable=0;}
//----------------------------------------------------------------------
unsigned char leddevnum; // Store LED1878 number
// Refresh led (must be called every 0.1s)
void leds_refresh()
  {unsigned char i;
   // Refresh indicator (on queue)
   if(is_mlan_done() && ledsenable)
     {// Prepare data      
      for(i=0;i<4;i++)
        {if(ledflash){TLED[i]=(LED_DATA[leddevnum].data[i])|(LED_DATA[leddevnum].data[i+4]);}
	 else{TLED[i]=(LED_DATA[leddevnum].data[i])&(~LED_DATA[leddevnum].data[i+4]);}
	}
      // Set transfer
      mlan_transfer(LED_DATA[leddevnum].adr,0xA5,0x04,LED_DATA[leddevnum].line,TLED,0x00);
      // Go to next indicator      
      leddevnum++;if(leddevnum>=LED_NUM){leddevnum=0;}
     }
   // Flash counter
   if(flashcnt){flashcnt--;}
   if(!flashcnt){ledflash=!ledflash;flashcnt=FLASH_PERIOD;}
  }
//----------------------------------------------------------------------
#endif
