//----------------------------------------------------------------------
//MODEULE: mlans.c
//COMMON DESCRIPTION:
//  This module contain procedures for transmiting and recieving data
//to and from one-wire bus (SLAVE MODE)
//----------------------------------------------------------------------
#include <avr/signal.h>
//----------------------------------------------------------------------
#ifdef MICROLAN_SLAVE_ENABLE
//----------------------------------------------------------------------
#include "mlans.h"
//----------------------------------------------------------------------
// Timing
// MICROLAN_SLAVE_F_TINT = interrupt frequency (With it frequency call mlan_int() procedure)
// All time intervals of one-wire protocol clculates from MICROLAN_SLAVE_F_TINT
#define MLANS_RESET_TIME  (unsigned char)(MICROLAN_SLAVE_F_TINT/1250)
#define MLANS_BIT_TIME    (unsigned char)(MICROLAN_SLAVE_F_TINT/2000)
#define MLANS_RECBIT_TIME (unsigned char)(MICROLAN_SLAVE_F_TINT/3333)
#define MLANS_SYNC_TIME   (unsigned char)(MICROLAN_SLAVE_F_TINT/10000)
// MLANS_CTRL CONSTANTS:
#define MLANS_STAY_MASK              0x07
#define MLANS_STAY_NONE              0x00
#define MLANS_STAY_RDADR             0x01
#define MLANS_STAY_RDCMD             0x02
#define MLANS_STAY_RDDATA            0x03
#define MLANS_STAY_WRDATA            0x04
// Bit 3: if =0, int enable, timer disable; else (if =1) timer enable, int disable
#define MLANS_NOINT_TIMER_BIT        0x03
#define MLANS_ENDTRS_BIT             0x04
//
// MLAN_CTRL FORMAT :
// Bits [2..0]: current operation
// [2..0]= 0 - none, must be recieving reset
// [2..0]= 1 - Adress reading
// [2..0]= 2 - Command reading
// [2..0]= 3 - Data reading
// [2..0]= 4 - Data writting
// Bit 3: if =0, int enable, timer disable; else (if =1) timer enable, int disable
// Bit 4: if=1, all transfer ended
unsigned char  MLANS_CTRL;            // Control byte
unsigned char  MLANS_BYTE_CNT;        // Counter of bytes
unsigned char  MLANS_BIT_CNT;         // Counter of bits
unsigned char  MLANS_TIMER;           // Timer (in 0.2*T), where T is time of bit transfer
unsigned char  MLANS_RESTIMER;        // Timer for reset
unsigned char  MLANS_WRITE;           //  Number of bytes of answer
unsigned char  MLANS_READ;            // Number of bytes of query
unsigned char  MLANS_BYTE;            // BYTE for MLAN reading
unsigned char  MLANS_COMMAND;         // Storage command
unsigned char  MLANS_RD_BUFFER[0x08]; // Buffer for reading
unsigned char  MLANS_WR_BUFFER[0x08]; // Buffer for writting
//----------------------------------------------------------------------
// Defines different macros
// ENABLE_TIMER() - enable timer and disable interrupt handler
#define MLANS_ENABLE_TIMER() MLANS_CTRL= MLANS_CTRL | (1 << MLANS_NOINT_TIMER_BIT);
// ENABLE_INT() - enable interrupt and disable timer handler
#define MLANS_ENABLE_INT()   MLANS_CTRL= MLANS_CTRL & (~(1 << MLANS_NOINT_TIMER_BIT));
// Is Valid adress ?
#define IS_ADRESS_VALID(a) a==MLANS_ADRESS
//----------------------------------------------------------------------
// Is Valid command ?
unsigned char IS_VALID_COMMAND()
  {unsigned char mret=0;
   // Standard commands
   if(MLANS_COMMAND==STD_CMD_RESET)
     {MLANS_READ=0x00;
      MLANS_WRITE=0x00;
      mret=1;      
     }
   else if(MLANS_COMMAND==STD_CMD_GETRDY)
     {MLANS_READ=0x00;
      MLANS_WRITE=0x00;
      mret=1;
     }
   else if(MLANS_COMMAND==STD_CMD_TST)
     {MLANS_READ=0x01;
      MLANS_WRITE=0x01;
      mret=1;
     }
   else 
     {// User commands
      #ifdef MLAN_SLAVE_U_ISVALCMD
        mret=MLAN_SLAVE_U_ISVALCMD(MLANS_COMMAND,&MLANS_READ,&MLANS_WRITE);
      #endif
     }
   //
   return(mret);
  }
//----------------------------------------------------------------------
// Command handler
void VALID_COMMAND_HDL()
  {// Standard commands
   if(MLANS_COMMAND==STD_CMD_RESET)
     {__asm__ __volatile__ ("rjmp 0" "\n\t");
     }
//   if(MLANS_COMMAND==STD_CMD_GETRDY)
//     {
//     }
   if(MLANS_COMMAND==STD_CMD_TST)
     {MLANS_WR_BUFFER[0x00]=MLANS_RD_BUFFER[0x00];      
     }
   else 
     {// User commands
      #ifdef MLAN_SLAVE_U_VALCMDHDL
        MLAN_SLAVE_U_VALCMDHDL(MLANS_COMMAND,MLANS_RD_BUFFER,MLANS_WR_BUFFER);
      #endif
     }
   //
  }
//----------------------------------------------------------------------
// Calculate CRC for MLANS_RD_BUFFER[DataLen]
unsigned char RD_CALC_CRC()
  {unsigned char i;
   unsigned char crc=0xAA;
   crc=crc+MLANS_ADRESS;
   crc=crc+MLANS_COMMAND;
   for(i=0;i<(MLANS_READ-1);i++)
     {crc=crc+MLANS_RD_BUFFER[i];
     }
   return(crc);
  }
//----------------------------------------------------------------------
// Calculate CRC and put crc to MLANS_WR_BUFFER[DataLen]
void WR_CALC_CRC()
  {unsigned char i;
   unsigned char crc=0xAA;
   for(i=0;i<MLANS_WRITE;i++)
     {crc=crc+MLANS_WR_BUFFER[i];
     }
   MLANS_WR_BUFFER[MLANS_WRITE]=crc;
   MLANS_WRITE++;
  }
//----------------------------------------------------------------------
// Initialize protocol MLAN
void MLANS_INIT()
  {output(MLANSPORT,input(MLANSPORT) | (1<<MLANSWRBIT));_NOP();_NOP();
   output(MLANSDDR,input(MLANSDDR)   | (1<<MLANSWRBIT));_NOP();_NOP();
   output(MLANSDDR,input(MLANSDDR)   & (~(1<<MLANSRDBIT)));_NOP();_NOP();
   MLANS_CTRL=MLANS_STAY_NONE;
  }
//----------------------------------------------------------------------
// MLAN Timer Interrupt handler
void MLANS_PTIMER()
  {unsigned char rdb=input(MLANSPIN);
   // if timer enable
   if(MLANS_CTRL & (1 << MLANS_NOINT_TIMER_BIT))
     {MLANS_TIMER++;      
      // Reading byte ?
      if(((MLANS_CTRL & MLANS_STAY_MASK)>=MLANS_STAY_RDADR) && 
         ((MLANS_CTRL & MLANS_STAY_MASK)<=MLANS_STAY_RDDATA))
        {if(MLANS_TIMER==MLANS_RECBIT_TIME)
	   {MLANS_BYTE=MLANS_BYTE >> 0x01;
	    if(rdb & (1<<MLANSRDBIT)){MLANS_BYTE=MLANS_BYTE | 0x80;}
	    else{MLANS_BYTE=MLANS_BYTE & 0x7F;}	    
	    MLANS_BIT_CNT++;    
	    if(MLANS_BIT_CNT>=0x08)
	      {MLANS_BIT_CNT=0x00;
	       // Reading adress ?
               if((MLANS_CTRL & MLANS_STAY_MASK)==MLANS_STAY_RDADR)
	         {// Yes, reding adress 
		  if(IS_ADRESS_VALID(MLANS_BYTE))
	            {// My Adress packet
		     #ifdef MLANS_U_READ_ADR_END
		       MLANS_U_READ_ADR_END();
		     #endif
		     //
	             MLANS_CTRL=(MLANS_CTRL & (~MLANS_STAY_MASK)) | MLANS_STAY_RDCMD;
	            }
	          else 
		    {// Not validate adress
		     MLANS_CTRL=MLANS_STAY_NONE;
		    }
		 }
	       // Reading command ?
               else if((MLANS_CTRL & MLANS_STAY_MASK)==MLANS_STAY_RDCMD)
                 {// Yes, reding command
		  MLANS_COMMAND=MLANS_BYTE;
	          if(IS_VALID_COMMAND())
	            {// Reading data (or crc only if no data)
	             MLANS_BYTE_CNT=0x00;
		     MLANS_READ++;
	             MLANS_CTRL=(MLANS_CTRL & (~MLANS_STAY_MASK)) | MLANS_STAY_RDDATA;
	            }
	          else
	            {// Not validate command
	             MLANS_CTRL=MLANS_STAY_NONE;
	            }
		 }
	       // Reading data ?
               else if((MLANS_CTRL & MLANS_STAY_MASK)==MLANS_STAY_RDDATA)
                 {MLANS_RD_BUFFER[MLANS_BYTE_CNT++]=MLANS_BYTE;
		  if(MLANS_BYTE_CNT>=MLANS_READ)
		    {// All data recieved		     
		     // CRC
		     if(RD_CALC_CRC()==MLANS_RD_BUFFER[MLANS_READ-0x01])
		       {VALID_COMMAND_HDL(); // Prepare data for transmitt
		        if(MLANS_WRITE)
		          {WR_CALC_CRC();
		           MLANS_BYTE_CNT=0x00;
		           MLANS_BYTE=MLANS_WR_BUFFER[MLANS_BYTE_CNT];
		           MLANS_CTRL=(MLANS_CTRL & (~MLANS_STAY_MASK)) | MLANS_STAY_WRDATA;			   
		          }
		        else
		          {// No data for transmitt
		           MLANS_CTRL=MLANS_STAY_NONE;
		          }
			//
			#ifdef MLANS_U_READ_DATA_END
			  MLANS_U_READ_DATA_END();
			#endif
			//
		       }
		     else
		       {// CRC Failed
		        MLANS_CTRL=MLANS_STAY_NONE;
		       }		     
		    }
	         }
	      }
	    MLANS_ENABLE_INT();
	   }
	}     
      // Writting byte ?
      else if((MLANS_CTRL & MLANS_STAY_MASK)==MLANS_STAY_WRDATA)
        {// Yes
	 if(MLANS_TIMER>=MLANS_BIT_TIME)
	   {output(MLANSPORT,input(MLANSPORT) | (1<<MLANSWRBIT));_NOP();_NOP();
            output(MLANSDDR,input(MLANSDDR) & (~(1<<MLANSRDBIT)));_NOP();_NOP();
	    if(MLANS_CTRL & (1<<MLANS_ENDTRS_BIT))
	      {MLANS_CTRL=MLANS_STAY_NONE;
	      }
	    MLANS_ENABLE_INT();
	   }
        }
      //----------------------------------------------------------------------      
     }
   // Reset ?
   if(!(rdb & (1<<MLANSRDBIT)))
     {MLANS_RESTIMER++;
      // Yes, reset!
      if(MLANS_RESTIMER>=MLANS_RESET_TIME)
        {MLANS_CTRL=MLANS_STAY_RDADR;
	 MLANS_BIT_CNT=0x00;
	 MLANS_ENABLE_INT();
	}
     }
   else
     {MLANS_RESTIMER=0x00;}
  }
//----------------------------------------------------------------------
// MLAN Interrupt handler
void MLANS_INT()
  {// if interrupt enable
   if(!(MLANS_CTRL & (1 << MLANS_NOINT_TIMER_BIT)))
     {// Writting byte ?
      if((MLANS_CTRL & MLANS_STAY_MASK)==MLANS_STAY_WRDATA)
        {// Yes, byte writting
	 // Setbit
	 if(MLANS_BYTE & 0x01)
	   {output(MLANSPORT,input(MLANSPORT) | (1<<MLANSWRBIT));_NOP();_NOP();}
	 else
	   {output(MLANSPORT,input(MLANSPORT) & (~(1<<MLANSWRBIT)));_NOP();_NOP();}
	 // Set output
	 output(MLANSDDR,input(MLANSDDR) | (1<<MLANSWRBIT));_NOP();_NOP();
	 // Next bit
	 MLANS_BYTE=MLANS_BYTE>>0x01;
	 MLANS_BIT_CNT++;         
	 if(MLANS_BIT_CNT>=0x08)
	   {MLANS_BIT_CNT=0x00;
	    MLANS_BYTE_CNT++;	    
	    MLANS_BYTE=MLANS_WR_BUFFER[MLANS_BYTE_CNT];
	    if(MLANS_BYTE_CNT>=MLANS_WRITE)
	      {MLANS_CTRL=MLANS_CTRL | (1<<MLANS_ENDTRS_BIT);
	       #ifdef MLANS_U_WRITE_DATA_END
	         MLANS_U_WRITE_DATA_END();
	       #endif
	      }	    
	   }	 
        }
      //
      MLANS_TIMER=0x00;
      MLANS_RESTIMER=0x00;
      MLANS_ENABLE_TIMER();      
     }
  }
//----------------------------------------------------------------------
#endif
//----------------------------------------------------------------------
